//
// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
//
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
//
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/runtime/c/ADLFactorySet.c,v $
// $Revision: 1.6 $
// $Date: 1998/07/21 13:42:51 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   C Runtime Environment - Factory set class
// NAME:        ADLFactorySet.c
//
// HISTORY:
//      Shane McCarron, The Open Group - 11 April 1998: Converted
//      Lionel Mallet, The Open Group RI  - Nov  2, 1997: Created
//      See Log at end of file
//
// ***********************************************************************

#include "ADLTest.h"
#include "ADLFactorySet.h"
#include "ADLResultCodes.h"
#include "ADLMemory.h"
#include <string.h>

static void *provide (ADLDataSet *d, int index) {
  int factoryStatus;

// check instance number
// checkBounds(index);

  d->mode.f.providedObject = NULL;

  if (ADL_debugMode) {
      ADL_printf("Provide for factory %s:", d->mode.f.factoryName);
  }

  // get the parameter values for the factory method		
  // 
  if (ADL__getNumElements(d->mode.f.params) > 0) {
	d->mode.f.factoryParameters = ADL__provide(d->mode.f.params, index);
  }
  
  if (d->mode.f.factoryProvide) {
      d->mode.f.providedObject = d->mode.f.factoryProvide(d->mode.f.factoryParameters);
  } else {
      return NULL;
  }

  if (ADL_skipAssertions) {
    // if tdd_skip has been called in provide, status is SKIPPED
    factoryStatus = ADL_SKIPPED;
  } else if (d->mode.f.providedObject != NULL) {
    // provide was successful
    factoryStatus = ADL_PASS;
  } else {
    // provide probably failed
    factoryStatus = ADL_FAIL;
    tdd_skip();
  }
  
  ADL_factoryResult(factoryStatus, "\tfactory returned: %d", factoryStatus);
  
  return d->mode.f.providedObject;
}

static int relinquish(ADLDataSet *d, int index) {
  int factoryStatus = 1;

  if (ADL_debugMode) {
      ADL_printf("Relinquish for factory %s", d->mode.f.factoryName);
  }

  if (d->mode.f.providedObject != NULL) {
    if (d->mode.f.factoryRelinquish != NULL) {
      factoryStatus = 
	d->mode.f.factoryRelinquish(d->mode.f.factoryParameters, d->mode.f.providedObject);
    }
    if (ADL__getNumElements(d->mode.f.params) > 0) {
      factoryStatus &= ADL__relinquish(d->mode.f.params, index);
    }

    /* in any case the provided object was malloc'ed by the generated code
       so free it */
    ADL__free(d->mode.f.providedObject);
    d->mode.f.providedObject = NULL;
  }
  
  if (d->mode.f.factoryParameters != NULL) {
    ADL__free(d->mode.f.factoryParameters);
    d->mode.f.factoryParameters = NULL;
  }

  if (ADL_debugMode) {
      ADL_printf("\tfactory returned: %d", factoryStatus);
  }

  return factoryStatus;
}

ADLDataSet *ADL_FactorySetInit(char *fname) {

    ADLDataSet *d;

    d = ADL__calloc(1, sizeof(ADLDataSet));
    if (d) {
	d->mode.f.params = ADL_ParameterSetInit();
	d->provide = provide;
	d->relinquish = relinquish;
	d->mode.f.factoryParameters = NULL;
	d->mode.f.providedObject = NULL;

	if (fname) {
	    d->mode.f.factoryName = ADL__new_string(fname);
	} else {
	    d->mode.f.factoryName = ADL__new_string("Anonymous");
	}
    }
    return d;
}

void ADL_FactoryAddParameter(ADLDataSet *f, ADLDataSet *e) {
  ADL__addElement(f->mode.f.params, e);
  f->numValues = ADL__getNumValues(f->mode.f.params);
}

int ADL_FactoryGetNumElements(ADLDataSet *f) {
  return ADL__getNumElements(f->mode.f.params);
}


// ***********************************************************************
// History Log:
// $Log: ADLFactorySet.c,v $
// Revision 1.6  1998/07/21 13:42:51  mallet
// Added test in relinquish() to avoid calling the factoryRelinquish
// function when there is none.
// Added call to ADL__free() to free the memory allocated in the
// generated factory code for the provided object.
// Changed factoryName initialization to use ADL__new_string().
//
// Revision 1.5  1998/05/11 12:54:08  shane
// Changed _ADL_* symbols to ADL__* since our naming convention says that all ADL
// symbols should start with ADL or tdd.
//
// Revision 1.4  1998/05/08 20:21:06  shane
// Various modifications, including:
//
// Changed to print certain information only when debugMode is true.
// Changed to actually set debugMode based upon ADL_RPT_MODE in configuration.
// Fixed how pointers are used (man am I stupid).
//
// Revision 1.3  1998/04/30 17:01:30  shane
// Fixed a few inconsistencies.
//
// Revision 1.2  1998/04/29 15:58:39  shane
// Fixed my stupid unions that didn't work right.
//
// Revision 1.1  1998/04/16 12:09:10  shane
// First checkin of complete set.
//
//
// ***********************************************************************
