//
// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
//
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
//
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/runtime/cpp/ADLConcatenationSet.hh,v $
// $Revision: 1.5 $
// $Date: 1998/02/27 10:01:02 $
// PRODUCT:	Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:	C++ Runtime Environment - Test directive and factory 
//                                        parameter set class
// NAME:	ADLConcatenationSet.hh
// 	
// HISTORY:
//      Lionel Mallet, The Open Group RI  - Oct  26, 1997: Created
//      See Log at end of file
// 		
// ***********************************************************************

#ifndef ADLConcatenationSet_hh
#define ADLConcatenationSet_hh

#include "ADLDataSet.hh"
#include "ADLRuntimeException.hh"
#include <stdlib.h>

/*
 *  class ADLConcatenationSet
 *
 *  The ADLConcatenationSet represents a TDD dataset which is defined
 *  as a concatenation of other TDD datasets. Its number of values is the
 *  sum of the number of values of its members.
 *
 */
class ADLConcatenationSet : public ADLDataSet {
public:
  ADLConcatenationSet(ADLTest *tc, char *label = "Anonymous") : 
    ADLDataSet(tc, label), elementSet(NULL), numElements(0), 
    providedSet(-1), providedIndex(-1) {}

  void addElement(ADLDataSet *e);

  void *provide(int index) throw (ADLRuntimeException);

  bool relinquish(int index) throw (ADLRuntimeException);
protected:

private:
  ADLDataSet **elementSet; // array of concatenated ADLDataSet
  int numElements; // number of ADLDataSet concatenated

  int providedSet; // element from which the value was provided
  int providedIndex; // index of the provided value in the element

  void checkBounds(int instance) throw (ADLRuntimeException);

  bool locateDataSet(int instance);
};
#endif // ADLConcatenationSet_hh

// ***********************************************************************
// History Log:
// $Log: ADLConcatenationSet.hh,v $
// Revision 1.5  1998/02/27 10:01:02  mallet
// Updated copyright notice.
//
// Revision 1.4  1998/02/19 09:54:39  mallet
// Updated checkBounds according to its new signature and semantics.
// Updated use of checkBounds in provide and relinquish according to its
// new signature.
//
// Revision 1.3  1998/02/12 16:04:25  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.2  1997/11/21 16:43:18  mallet
// Replaced string by char* to become more portable.
//
// Revision 1.1  1997/11/14 10:13:28  mallet
// Initial version of runtime classes for TDD datasets and ACOs.
//
//
// ***********************************************************************
