//
// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
//
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
//
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/runtime/cpp/ADLLiteralSet.hh,v $
// $Revision: 1.10 $
// $Date: 1998/03/03 22:31:27 $
// PRODUCT:	Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:	C++ Runtime Environment - Test directive and factory 
//                                        parameter set class
// NAME:	ADLLiteralSet.hh
// 	
// HISTORY:
//      Lionel Mallet, The Open Group RI  - Oct  28, 1997: Created
//      See Log at end of file
// 		
// ***********************************************************************

#ifndef ADLLiteralSet_hh
#define ADLLiteralSet_hh

#include "ADLDataSet.hh"
#include "ADLRuntimeException.hh"
#include <stdlib.h>

#ifdef WIN32
#include <memory.h>
#endif

/*
 *  class ADLLiteralSet
 *
 *  The ADLLiteralSet represents a TDD dataset which is defined
 *  as a concatenation of other TDD datasets. Its number of values is the
 *  sum of the number of values of its members.
 *
 */
class ADLLiteralSet : public ADLDataSet {
protected:
  void *values; // array of values
  // Note: we could have used a union of the different builtin types of
  // C++ as the type of each value but this would have resulted in a huge
  // waste of memory space since each value would have taken the size of
  // a long double, which, in most cases, it doesn't use.
  // Considering each value as a void* is the best memory usage
  // compromise - at least until 64bits machines are common. 

  short typeSize;

  void setValues(ADLLiteralSet *);

private:
  static short chunkNumberOfValues;
  short maxNumberOfValues;

  void doAddValue(void *);

public:

  ADLLiteralSet(ADLTest *tc) : ADLDataSet(tc), values(NULL),
    typeSize(0), maxNumberOfValues(0) {}

  ADLLiteralSet(ADLTest *tc, short typeSiz) : ADLDataSet(tc),
    values(NULL), typeSize(typeSiz), maxNumberOfValues(0) {}

#ifdef __GNUG__
  ADLLiteralSet(ADLTest *, bool [], int);
#endif
  ADLLiteralSet(ADLTest *, char [], int);
  ADLLiteralSet(ADLTest *, unsigned char [], int);
  ADLLiteralSet(ADLTest *, short [], int);
  ADLLiteralSet(ADLTest *, unsigned short [], int);
  ADLLiteralSet(ADLTest *, int [], int);
  ADLLiteralSet(ADLTest *, unsigned int [], int);
  ADLLiteralSet(ADLTest *, long [], int);
  ADLLiteralSet(ADLTest *, unsigned long [], int);
  ADLLiteralSet(ADLTest *, float [], int);
  ADLLiteralSet(ADLTest *, double [], int);
  ADLLiteralSet(ADLTest *, long double [], int);
  ADLLiteralSet(ADLTest *, void *[], int);

#ifdef __GNUG__
  void addValue(bool) throw (ADLRuntimeException);
#endif
  void addValue(char) throw (ADLRuntimeException);
  void addValue(unsigned char) throw (ADLRuntimeException);
  void addValue(short) throw (ADLRuntimeException);
  void addValue(unsigned short) throw (ADLRuntimeException);
  void addValue(int) throw (ADLRuntimeException);
  void addValue(unsigned int) throw (ADLRuntimeException);
  void addValue(long) throw (ADLRuntimeException);
  void addValue(unsigned long) throw (ADLRuntimeException);
  void addValue(float) throw (ADLRuntimeException);
  void addValue(double) throw (ADLRuntimeException);
  void addValue(long double) throw (ADLRuntimeException);
  void addValue(void *) throw (ADLRuntimeException);
  void addRangeValues(char, char)
    throw (ADLRuntimeException);
  void addRangeValues(unsigned char, unsigned char)
    throw (ADLRuntimeException);
  void addRangeValues(short, short)
    throw (ADLRuntimeException);
  void addRangeValues(unsigned short, unsigned short)
    throw (ADLRuntimeException);
  void addRangeValues(int, int)
    throw (ADLRuntimeException);
  void addRangeValues(unsigned int, unsigned int)
    throw (ADLRuntimeException);
  void addRangeValues(long, long)
    throw (ADLRuntimeException);
  void addRangeValues(unsigned long, unsigned long)
    throw (ADLRuntimeException);

  void *provide(int index) throw (ADLRuntimeException);
  bool relinquish(int index) throw (ADLRuntimeException);
};
#endif // ADLLiteralSet_hh

// ***********************************************************************
// History Log:
// $Log: ADLLiteralSet.hh,v $
// Revision 1.10  1998/03/03 22:31:27  shane
// Added handling for WIN32-isms.
// Now compiles cleanly on Win32 - don't know if it runs.
//
// Revision 1.9  1998/03/03 10:46:26  mallet
// Optimized doAddValue by allocating chunks of chunkNumberOfValues
// instead of just one value. This reduces the number of
// malloc/realloc. chunkNumberOfValues is initially set to 5. This value
// would have to be refined according to some measurements of the mean
// number of values in a literal dataset in ADL test suites.
//
// Revision 1.8  1998/02/27 10:01:04  mallet
// Updated copyright notice.
//
// Revision 1.7  1998/02/26 10:22:35  mallet
// Added the addValue methods for all builtin types to allow to add a
// value without passing its address: this is more robust since it is
// possible to check that the size of the type of the added value is
// compatible with the size of the other values in the dataset.
// Renamed previous addValue(void *) as doAddValue(void *): use this to
// finally add a value in addValue and addRangeValues methods.
//
// Revision 1.6  1998/02/26 09:22:12  mallet
// Made addValue method public.
// Added missing contructors for some builtin types (unsigned ..., long
// double).
// Added addRangeValues methods for integral types (unsigned char -> long).
//
// Revision 1.5  1998/02/19 09:59:06  mallet
// Added missing initializations in ADLLiteralSet(ADLTest) constructor:
// values and typeSize.
// Added new constructor ADLLiteralSet(ADLTest, short) to create a
// literal set with no initial value (see addValue).
// Added a new method addValue to add a new value to an ADLLiteralSet:
// note that no check is done on the size of the passed value w.r.t. the
// size of the expected value for the literal set.
// Changed signature of provide and relinquish to throw an
// ADLRuntimeException when checkBounds fails.
// Changed use of checkBounds in provide and relinquish according to its
// new signature.
//
// Revision 1.4  1998/02/12 16:04:27  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.3  1998/01/29 12:53:46  bareau
// Update of the C++ runtime.
// In ADLFactorySet, factory_relinquish is no more virtual
// but has a default behavior "return true".
// In ADLLiteralSet, added a default constructor with one parameter
// ADLTest *, and a method "void setValues(ADLLiteralSet* copySet)"
// to create an ADLLiteralSet from an other one.
//
// Revision 1.2  1997/11/21 16:56:10  mallet
// Added an int size parameter to constructors since the size of the
// array cannot be dynamically computed.
// Bool constructor is surrounded by #ifdef __GNUG__ since it seems the
// g++ is the only compiler to define the bool type yet: for other
// compilers the bool type is often defined as int.
//
// Revision 1.1  1997/11/14 10:13:30  mallet
// Initial version of runtime classes for TDD datasets and ACOs.
//
//
// ***********************************************************************
