//
// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
//
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
//
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/runtime/cpp/ADLParameterSet.cpp,v $
// $Revision: 1.5 $
// $Date: 1998/02/27 10:01:04 $
// PRODUCT:	Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:	C++ Runtime Environment - Test directive and factory 
//                                        parameter set class
// NAME:	ADLParameterSet.hh
// 	
// HISTORY:
//      Lionel Mallet, The Open Group RI  - Oct  26, 1997: Created
//      See Log at end of file
// 		
// ***********************************************************************

#include "ADLParameterSet.hh"
#include "ADLString.hh"
ADLParameterSet::ADLParameterSet() : elementSet(NULL), numElements(0), numValues(1), elementIndexes(NULL) {}

void ADLParameterSet::addElement(ADLDataSet *e) {
  // alloc a new element set with one more element
  ADLDataSet **newSet = new ADLDataSet *[numElements + 1];

  if (numElements > 0) {
    // copy elements of old set into new one
    for (int i = 0; i < numElements; i++) {
      newSet[i]  = elementSet[i];
    }
    // now free old set
    delete elementSet;
  }

  // now set last element and set element set to newly allocated one
  newSet[numElements] = e;
  elementSet = newSet;

  // finally increase number of elements and values
  numElements += 1;
  numValues *= e->getNumValues();
}

void **ADLParameterSet::provide(int index) throw (ADLRuntimeException) {

  checkBounds(index);

  // free previously alloced array if not null
  if (elementIndexes != NULL) delete elementIndexes;
  elementIndexes = computeIndexes(index);

  // allocate returned array (freed by caller)
  void **ret = new void *[numElements];

  for (int x = 0; x < numElements; x++) {
    ADLDataSet *data = elementSet[x];
    ret[x] = (void *)data->provide(elementIndexes[x]);
  }

  return ret;
}

bool ADLParameterSet::relinquish(int index) throw (ADLRuntimeException) {
  
  checkBounds(index);

  bool result = true;

  for (int x = 0; x < numElements; x++) {
    ADLDataSet *data = elementSet[x];
    result &= data->relinquish(elementIndexes[x]);
  }

  return result;
}

int *ADLParameterSet::computeArrayofMaxInstances() {
  // allocate returned array (freed by caller)
  int *elementInstanceValues = new int[numElements];

  for (int x = 0; x < numElements; x++) {
    ADLDataSet *data = elementSet[x];
    elementInstanceValues[x] = data->getNumValues();
  }
  return elementInstanceValues;
}

// Note that instance number should be starting at 0
int *ADLParameterSet::computeIndexes(int instance)
  throw (ADLRuntimeException) {

  checkBounds(instance);

  if (numElements == 0) {
    return NULL;
  }

  // compute the array of element max instance values (freed at end of
  // method)
  int *maxInstanceArray = computeArrayofMaxInstances();

  // allocate array of computed indexes (never freed in provide)
  int *ret = new int[numElements];

  // perform magic computation
  for (int j = numElements - 1; j >= 0; j--) {
    ret[j] = (instance % maxInstanceArray[j]);
    instance /= maxInstanceArray[j];
  }

  // free array of max instances
  delete maxInstanceArray;

  // final check
  if (instance != 0) {
    throw ADLRuntimeException(ADLRuntime, 
			      "ADLParameterSet: instance != 0");
  }

  return ret;
}

void ADLParameterSet::checkBounds(int index) throw (ADLRuntimeException) {
  if (index < 0 || index >= numValues) {
    throw ADLRuntimeException(ADLRuntime,
			      adlstr.reset()
			      << "ADLParameterSet: index out of range ("
			      << index << "/" << numValues << ")");
  }
}

// ***********************************************************************
// History Log:
// $Log: ADLParameterSet.cpp,v $
// Revision 1.5  1998/02/27 10:01:04  mallet
// Updated copyright notice.
//
// Revision 1.4  1998/02/19 09:50:38  mallet
// Changed signature and semantics of checkBounds to throw an
// ADLRuntimeException instead of returning a boolean.
// Changed signature of computeIndexes, provide and relinquish to throw an
// ADLRuntimeException when checkBounds fails instead of returning NULL,
// which could be gently ignored.
// Changed usage of checkBounds in computeIndexes, provide and relinquish
// according to its new signature.
//
// Revision 1.3  1998/02/12 16:04:28  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.2  1997/11/21 16:56:32  mallet
// Fixed use of new for array of pointers.
//
// Revision 1.1  1997/11/14 10:13:30  mallet
// Initial version of runtime classes for TDD datasets and ACOs.
//
//
// ***********************************************************************
