#include	<string.h>

int	adl_optind;	/*index of the current argument (0 before first call)*/

int	adl_optopt;	/*current option letter*/

char	*adl_optarg;	/*pointer to the option-associated argument*/

int	adl_getopt (int argc, char* const argv[], char* defined_options_p);

static void  skip_to_next_options_argument (int argc, char * const argv []);

static	char	*arg_p;			/* pointer to current argument */

static	int	forward_args_used;	/* number of associated forward args already used */

static	int	used_columns;		/* columns used from current argument */


/*
// This procedure parses the next option in a given list of command line
// options. The allowable options are given in the "defined_options_p" string,
// in the form:
//
//		"A:bcdef:ghH"  ...
//
// where each letter in the string is an allowed option letter, and a colon
// following an option letter means that an associated following argument is required.
*/

int  adl_getopt (int argc, char *const argv[], char *defined_options_p)
{
	int		arg_index;
	char	expected_option;
	char	*options_p;
	char	*p;

   	adl_optarg = NULL;

	if (adl_optind == 0)
		skip_to_next_options_argument (argc, argv);

  	while (adl_optind < argc) {

		if (arg_p == NULL)
			return -1;

		p = arg_p + used_columns;

		if ((adl_optopt = *p++) == 0) {
			skip_to_next_options_argument (argc, argv);
			continue;
		}

		++used_columns;

		/* Look up the found "optopt" option letter in the table of defined */
		/* options. */

		options_p = defined_options_p;

		while ((expected_option = *options_p++) != 0) {

			if (expected_option == ':') continue;	/* colon means argument expected to follow */

			if (adl_optopt == expected_option) {

				if (*options_p == ':') {		/* argument required */

					if (*p != 0) {
						adl_optarg = p;
						used_columns = strlen (arg_p);

					} else {
						arg_index = adl_optind + forward_args_used + 1;	/* expected index of argument */

						if (arg_index >= argc || * (argv [arg_index]) == '-') {
							adl_optarg = NULL;
							return((int)':');
						} else {
							adl_optarg = (char *) argv [arg_index];
							++ forward_args_used;
						}
					}

				}

				return adl_optopt;
			}
		}

		return '?';
	}

	return -1;


} /* getopt */

static void  skip_to_next_options_argument (int argc, char * const argv [])

/*
// This procedure advances "optind" and "arg_p" to the next argument containing
// options. If there is no such argument, then "optind" is set to the index of
// the very next argument number (possibly equal to "argc", i.e. one higher than
// the index of the final argument) and "arg_p" is set to NULL.
*/

{
	int		index;

	used_columns = 0;

	adl_optind += forward_args_used;

	forward_args_used = 0;

	index = adl_optind++;

	do {
		if (++index >= argc) {
			arg_p = NULL;
			return;
		}

		arg_p = argv[index];

	} while (* arg_p != '-');

	adl_optind = index;
	used_columns = 1;

	return;

} /* skip_to_next_options_argument */
